﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;
using SrPagoApi;
using SrPagoApi.Response;

namespace SrEcommerceDemo
{
    public partial class ChargeExample : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        [WebMethod]
        public static string MakePaymentAction(string TokenCard, string amount, string description)
        {
            var charge = Task.Run(async () =>
            {
                var result = await MakePaymentWithToken(TokenCard, amount, description);

                return result;
            }).GetAwaiter().GetResult();

            if ( charge.error != null)
            {
                return "Error: " + charge.error.message;
            }else
            {
                return  "Success AuthCode: " + charge.result.authorization_code;
            }
        }

        public static async Task<ChargesResponse> MakePaymentWithToken(string token, string amount, string description)
        {
            if (!decimal.TryParse(amount, out decimal decAmount))
            {
                ChargesResponse error = new ChargesResponse
                {
                    error = new Error
                    {
                        message = "Amount string cannot be parsed"
                    },
                    success = false
                };
                return error;
            }

            try
            {
                SrPago.ApiKey = ConfigurationManager.AppSettings.Get("ApiKey");
                SrPago.ApiSecret = ConfigurationManager.AppSettings.Get("ApiSecret");
                SrPago.LiveMode = Boolean.Parse(ConfigurationManager.AppSettings.Get("LiveMode"));

                // El token es creado usando SrPago.js o aplicaciones (https://js.srpago.com/v1/srpago.min.js)
                // Envia el token a via formulario, no requiere PCI Compliant, el sitio debe contar con certificado https

                Dictionary<string, object> chargeParams = new Dictionary<string, object>
                {
                    { "amount", decAmount },
                    { "description", description },
                    { "reference", "ABC123" },
                    { "source", token },
                    { "metadata", mapToMetadata() }

                    //{ "amount", decAmount },
                    //{ "description", description },
                    //{ "reference", "ABC123" },
                    //{ "source", token },
                    //{ "metadata", mapToMetadata() },
                    //{ "campoExtra1", 1 },
                    //{ "campoExtra2", "2" },
                    //{ "campoExtra3", 34},
                    //{ "lista", new List<int>{ 4, 34, 56, 90 } },
                    //{ "UsuarioExtra", new UserAux(45, "Daniel")},
                    //{ "ArrUsuarios", new List<UserAux>
                    //    {
                    //        new UserAux(78, "Jose"),
                    //        new UserAux(98, "Diego")
                    //    }
                    //}
                };

                ChargesService charges = new ChargesService();
                ChargesResponse charge = await charges.Create(chargeParams);

                if (charge.success)
                {
                    //Exito
                    Operation result = charge.result;
                }
                else
                {
                    //Error
                    string code = charge.error.code;
                    string message = charge.error.message;
                }

                return charge;
            }
            catch (InvalidParamException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (SrPagoException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }

            return null;

        }


        private static string mapToMetadata()
        {
            return @"{
					    ""salesTax"": ""160.00"",
					    ""browserCookie"": ""session=743663;"",
					    ""orderMessage"": ""No tocar el timbre, esperare afuera"",
					    ""billing"": {
					        ""billingEmailAddress"": ""johnsmith@email.com"",
					        ""billingFirstName-D"": ""Juan"",
					        ""billingMiddleName-D"": ""Antonio"",
					        ""billingLastName-D"": ""Perez"",
					        ""billingAddress-D"": ""Arkansas 16"",
					        ""billingAddress2-D"": ""Insurgentes"",
					        ""billingCity-D"": ""CDMX"",
					        ""billingState-D"": ""DF"",
					        ""billingPostalCode-D"": ""03810"",
					        ""billingCountry-D"": ""MEX"",
					        ""billingPhoneNumber-D"": ""87654321"",
					        ""creditCardAuthorizedAmount-D"": ""2000.00""
					    },
					    ""giftCards"": {""giftCard"": [
					        {
					            ""giftCardNumber"": ""ABC1234567890"",
					            ""giftCardAmount"": ""10.95""
					        },
					        {
					            ""giftCardNumber"": ""JKL09876543231"",
					            ""giftCardAmount"": ""15.30""
					        }
					    ]},
					    ""coupons"": {""coupon"": [
					        {
					            ""couponCode"": ""011293674310"",
					            ""couponAmount"": ""50.00""
					        },
					        {
					            ""couponCode"": ""704757305096"",
					            ""couponAmount"": ""70.00""
					        }
					    ]},
					    ""promotions"": {""promotion"": [
					        {
					            ""promotionCode"": ""P0001"",
					            ""promotionAmount"": ""100.00""
					        },
					        {
					            ""promotionCode"": ""P0002"",
					            ""promotionAmount"": ""50.00""
					        }
					    ]},
					    ""certificates"": {""certificate"": [{
					        ""certificateUsedValue"": ""76"",
					        ""certificatePromoCode"": ""07689"",
					        ""certificatePinCode"": ""65588"",
					        ""certificateLoyaltyMemberId"": ""88766445"",
					        ""certificateLoyaltyProgramCode"": ""867e548"",
					        ""certificateIssueReasonCode"": ""asdfg"",
					        ""certificateIssueLocation"": ""web"",
					        ""certificateIssueDate"": ""2015-07-03"",
					        ""certificateInitialValue"": ""100"",
					        ""certificateExpirationDate"": ""2016-01-01"",
					        ""certificateId"": ""66769"",
					        ""certificateCurrentValue"": ""24""
					    }]},
					    ""member"": {
					        ""memberLoggedIn"": ""Si/No"",
					        ""memberId"": ""TBI23457"",
					        ""membershipDate"": ""2010-02-12"",
					        ""memberFullName"": ""Berenice Garcia Toral"",
					        ""memberFirstName"": ""Berenice"",
					        ""memberMiddleName"": ""Garcia"",
					        ""memberLastName"": ""Toral"",
					        ""memberEmailAddress"": ""bgarcia@gmail.com"",
					        ""memberAddressLine1"": ""123 Main St"",
					        ""memberAddressLine2"": ""main"",
					        ""memberCity"": ""Chicago"",
					        ""memberState"": ""IL"",
					        ""memberCountry"": ""US"",
					        ""memberPostalCode"": ""60601"",
					        ""membershipLevel"": ""3"",
					        ""membershipStatus"": ""activo"",
					        ""latitude"": ""19"",
					        ""longitude"": ""19"",
					        ""memberPhone"": ""312-555-5555""
					    },
					    ""shipping"": {
					        ""shippingCharges"": ""100.50"",
					        ""shippingFirstName"": ""Juan"",
					        ""shippingMiddleName"": ""Pablo"",
					        ""shippingLastName"": ""Perez"",
					        ""shippingEmailAddress"": ""a@a.com"",
					        ""shippingAddress"": ""a"",
					        ""shippingAddress2"": ""b"",
					        ""shippingCity"": ""s"",
					        ""shippingState"": ""Chicago"",
					        ""shippingPostalCode"": ""12345"",
					        ""shippingCountry"": ""MEX"",
					        ""shippingPhoneNumber"": ""87654321"",
					        ""shippingMethod"": ""mail"",
					        ""shippingDeadline"": ""2015-08-01""
					    },
					    ""items"": {""item"": [
					        {
					            ""itemNumber"": ""193487654"",
					            ""itemDescription"": ""iPhone 6 32gb"",
					            ""itemPrice"": ""599.00"",
					            ""itemQuantity"": ""1"",
					            ""itemMeasurementUnit"": ""Pza"",
					            ""itemBrandName"": ""Apple"",
					            ""itemCategory"": ""Electronics"",
					            ""itemTax"": ""12.95""
					        },
					        {
					            ""itemNumber"": ""193487654"",
					            ""itemDescription"": ""iPhone 6 64gb"",
					            ""itemPrice"": ""599.00"",
					            ""itemQuantity"": ""2"",
					            ""itemMeasurementUnit"": ""Pza"",
					            ""itemBrandName"": ""Samsung"",
					            ""itemCategory"": ""Electronics"",
					            ""itemTax"": ""12.95""
					        }
					    ]},
					    ""dispatchers"": {""dispatcher"": [{
					        ""dispatcherId"": ""193487654"",
					        ""dispatcherName"": ""Jose"",
					        ""dispatcherLastName"": ""Mtz""
					    }]},
					    ""retail"": {
					        ""initialSalesRep"": ""Jaime Mtz"",
					        ""initialSalesRepDeviceId"": ""43r2e02dkewf2"",
					        ""initialSalesRepId"": ""1314"",
					        ""finalSalesRep"": ""Jaime Mtz"",
					        ""finalSalesRepDeviceId"": ""43r2e02dkewf2"",
					        ""finalSalesRepId"": ""1314"",
					        ""salesChannel"": ""Abarrotes, Vinos, Licores, "",
					        ""referralCode"": ""CFB2010""
					    },
					    ""inauth"": {""inauthTransactionId"": ""<ESTA INFORMACIÓN TENDRÁ AL INTEGRAR SERVICIO ADICIONAL>""}
					}
					";
        }
    }

    class UserAux
    {
        public int Id { get; set; }
        public string Name { get; set; }

        public UserAux(int id, string name)
        {
            Id = id;
            Name = name;
        }
    }
}