﻿using System;
using System.Configuration;
using System.Threading.Tasks;
using System.Web;
using System.Web.Services;
using System.Web.UI;
using SrPagoApi;
using SrPagoApi.Response;

namespace SrEcommerceDemo.Customer.Card
{

    public partial class Create : System.Web.UI.Page
    {
        private static string customerId;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(Request.QueryString["id"]))
            {
                string id = Request.QueryString["id"];
                var customer = Task.Run(async () =>
                {
                    var result = await GetCustomer(id);

                    return result;
                }).GetAwaiter().GetResult();
                lbCustomerName.Text = customer.result.name;
                customerId = customer.result.id;
            }
        }


        [WebMethod]
        public static string PostCustomerCardAction(string TokenCard)
        {

            var charge = Task.Run(async () =>
            {
                var result = await PostCustomerCard(TokenCard, customerId);

                return result;
            }).GetAwaiter().GetResult();

            if (charge.error != null)
            {
                return "Error: " + charge.error.message;
            }
            else
            {
                return "Success";
            }
        }

        public static async Task<CustomerResponse> GetCustomer(string id)
        {
            try
            {
                SrPago.ApiKey = ConfigurationManager.AppSettings.Get("ApiKey");
                SrPago.ApiSecret = ConfigurationManager.AppSettings.Get("ApiSecret");
                SrPago.LiveMode = Boolean.Parse(ConfigurationManager.AppSettings.Get("LiveMode"));

                CustomerService customerService = new CustomerService();
                CustomerResponse customer = await customerService.Retreive(id);
                return customer;
            }
            catch (InvalidParamException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (SrPagoException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }

            return null;
        }

        public static async Task<CustomerCardResponse> PostCustomerCard(string token, string id)
        {
            try
            {
                SrPago.ApiKey = ConfigurationManager.AppSettings.Get("ApiKey");
                SrPago.ApiSecret = ConfigurationManager.AppSettings.Get("ApiSecret");
                SrPago.LiveMode = Boolean.Parse(ConfigurationManager.AppSettings.Get("LiveMode"));

                CustomerService customerService = new CustomerService();
                CustomerCardResponse customerCard = await customerService.PostCustomerCard(token, id);
                return customerCard;
            }
            catch (InvalidParamException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (SrPagoException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }

            return null;
        }
    }
}
