﻿using System;
using System.Configuration;
using System.Threading.Tasks;
using System.Web;
using System.Web.UI;
using SrPagoApi;
using SrPagoApi.Response;

namespace SrEcommerceDemo.Customer
{

    public partial class View : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(Request.QueryString["id"]))
            {
                string id = Request.QueryString["id"];
                var customer = Task.Run(async () =>
                {
                    var result = await GetCustomer(id);

                    return result;
                }).GetAwaiter().GetResult();
                lbName.Text = customer.result.name;
                lbEmail.Text = customer.result.email;
                lbActive.Text = customer.result.active ? "SI" : "NO";

                gvCards.DataSource = customer.result.cards;
                gvCards.DataBind();

            }
        }

        public static async Task<CustomerResponse> GetCustomer(string id)
        {
            try
            {
                SrPago.ApiKey = ConfigurationManager.AppSettings.Get("ApiKey");
                SrPago.ApiSecret = ConfigurationManager.AppSettings.Get("ApiSecret");
                SrPago.LiveMode = Boolean.Parse(ConfigurationManager.AppSettings.Get("LiveMode"));

                CustomerService customerService = new CustomerService();
                CustomerResponse customer = await customerService.Retreive(id);
                return customer;
            }
            catch (InvalidParamException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (SrPagoException ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine(ex.StackTrace);
            }

            return null;
        }
    }
}